#include "testlib.h"
#include <string>
#include <vector>
#include <cmath>

using namespace std;

const long double EPS = 1e-2;

long double my_stold(const string& str, InStream& error_stream)
{
	size_t sz;
	long double res;
	try
	{
		res = stold(str, &sz);
	}
	catch (const invalid_argument& e)
	{
		error_stream.quitf(_pe, "Cannot convert %s to double (line %d)", str.c_str(), __LINE__);
	}
	catch (const out_of_range& e)
	{
		error_stream.quitf(_pe, "Cannot convert %s to double (line %d)", str.c_str(), __LINE__);
	}
	if (sz != str.length())
		error_stream.quitf(_pe, "Extra characters in double %s (line %d)", str.c_str(), __LINE__);
	return res;
}

vector <pair <bool, pair <long double, long double>>> parse(InStream& stream, int n)
{
    stream.maxMessageLength = 8388608;
	string s = stream.readString("[0123456789.+-x\\|]{1,8388608}", "s");
	int len = (int)s.length();
	
	int cnt = 0, i = 0, g = 0;
	vector <pair <bool, pair <long double, long double>>> res(n);
	res[g].first = true;
	if (s[i] == '-')
	{
		res[g].first = false;
		++i;
	}

	while (i < len)
	{
		if (s[i] != '|')
			stream.quitf(_pe, "Output format is wrong at %d symbol %c (line %d)", i, s[i], __LINE__);
		++i;
		++cnt;
		
		if (i == len)
			stream.quitf(_pe, "Output format is wrong at %d (line %d)", i, __LINE__);
		int num_start = i;
		if (s[i] == '-')
			++i;
		while (i < len && ((s[i] >= '0' && s[i] <= '9') || s[i] == '.'))
			++i;
		if (i == len || s[i] != 'x')
			stream.quitf(_pe, "Output format is wrong at %d (line %d)", i, __LINE__);
		res[g].second.first = my_stold(s.substr(num_start, i - num_start), stream);
		
		++i;
		if (s[i] == '+')
			++i;
		num_start = i;
		if (s[i] == '-')
			++i;
		while (i < len && ((s[i] >= '0' && s[i] <= '9') || s[i] == '.'))
			++i;
		if (i == len || s[i] != '|')
			stream.quitf(_pe, "Output format is wrong at %d (line %d)", i, __LINE__);
		res[g].second.second = my_stold(s.substr(num_start, i - num_start), stream);
		++i;
		++cnt;
		
		++g;
		if (cnt == 2 * n && i < len)
			stream.quitf(_pe, "Output format is wrong at %d (line %d)", i, __LINE__);
		if (i == len && cnt != 2 * n)
			stream.quitf(_pe, "Output format is wrong at %d (line %d)", i, __LINE__);

		if (i < len) {
			if (s[i] == '+')
				res[g].first = true;
			else if (s[i] == '-')
				res[g].first = false;
			else
				stream.quitf(_pe, "Output format is wrong at %d symbol %c (line %d)", i, s[i], __LINE__);
			++i;
		}
	}
	
	return res;
}

long double calc_in_dots(const vector<pair<int, int>>& dots, long double x)
{
	int x_rounded_down = x >= 0 ? (int)(x + 1e-12) : (int)(x + 1e-12) - 1;
	auto it = lower_bound(dots.begin(), dots.end(), make_pair(x_rounded_down, 1000000007));
	if (it == dots.begin())
	{
		if (x + 1e-9 < dots.front().first)
			throw logic_error("out of range of segment-linear function");
		return dots.front().second;
	}
	else if (it == dots.end())
	{
		if (x - 1e-9 > dots.back().first)
			throw logic_error("out of range of segment-linear function");
		return dots.back().second;
	}
	else
	{
		long double x1, y1, x2, y2;
		tie(x1, y1) = *prev(it);
		tie(x2, y2) = *it;
		return y1 + (x - x1) * (y2 - y1) / (long double)(x2 - x1);
	}
}

long double calc_in_modules(const vector<pair<bool, pair<long double, long double>>>& par, long double x)
{
	long double res = 0;
	for (const auto& e : par)
	{
		long double cur = abs(e.second.first * x + e.second.second);
		if (e.first)
			res += cur;
		else
			res -= cur;
	}
	return res;
}

const int ITER = 100;

void ONlogN_check(int n,
				  const vector <pair <int, int>>& dots,
				  const vector <pair <bool, pair <long double, long double>>>& par,
				  InStream& stream)
{
	vector<pair<long double, long double>> break_points;
	long double starting_derivative = 0;
	for (int i = 0; i < n; i++)
	{
		break_points.emplace_back(dots[i].first, 0);
		if (abs(par[i].second.first) < 1e-9)
			continue;
		long double cur_x = -par[i].second.second / par[i].second.first;
		if (dots.front().first <= cur_x && cur_x <= dots.back().first)
		{
			break_points.emplace_back(cur_x, abs(par[i].second.first) * (par[i].first ? 2 : -2));
			starting_derivative += abs(par[i].second.first) * (par[i].first ? -1 : 1);
		}
		else if (cur_x >= dots.back().first)
			starting_derivative += abs(par[i].second.first) * (par[i].first ? -1 : 1);
		else if (cur_x <= dots.front().first)
			starting_derivative += abs(par[i].second.first) * (par[i].first ? 1 : -1);
	}
	
	sort(break_points.begin(), break_points.end());
	int iter = min(ITER, (int)break_points.size());
	for (int i = 0; i < iter; ++i)
	{
		int from = i * (int)break_points.size() / iter;
		int to = (i + 1) * (int)break_points.size() / iter;
		long double derivative = starting_derivative;
		for (int j = 0; j < from; ++j)
			derivative += break_points[j].second;
		long double x = break_points[from].first;
		long double y = calc_in_modules(par, x);

		for (int j = from; j < to; ++j)
		{
			y += derivative * (break_points[j].first - x);
			x = break_points[j].first;
			derivative += break_points[j].second;
			long double y2 = calc_in_dots(dots, x);
			if (abs(y - y2) > EPS)
				stream.quitf(_wa, "answer f(%.6Lf)=%.6Lf != %.6Lf ", x, y, y2);
		}
	}
}

int main(int argc, char * argv[])
{
	registerTestlibCmd(argc, argv);
	
	int n = inf.readInt();
	vector <pair <int, int>> dots(n);
	for (int i = 0; i < n; i++)
	{
		dots[i].first = inf.readInt();
		dots[i].second = inf.readInt();
	}
	
	auto par1 = parse(ans, n);
	auto par2 = parse(ouf, n);
	
	ONlogN_check(n, dots, par1, ans);
	ONlogN_check(n, dots, par2, ouf);
	quitf(_ok, "OK");
	
	return 0;
}